<?php
namespace Drupal\pdir\Plugin\Filter;

use Drupal\filter\FilterProcessResult;
use Drupal\filter\Plugin\FilterBase;

/**
 * @Filter(
 *   id = "filter_pdir",
 *   title = @Translation("Determines the directionality of a paragraph"),
 *   description = @Translation("Sets the directionality of a paragraph, according to it's first alphabetic chracter. <strong>This filter should be generally used after the Line break converter filter</strong> in order for it to work."),
 *   type = Drupal\filter\Plugin\FilterInterface::TYPE_TRANSFORM_REVERSIBLE,
 *   weight = 999
 * )
 */
class FilterPdir extends FilterBase {
    public function process($text, $langcode):FilterProcessResult {
        // Find first text within a paragraph (ignore html inside paragraphs)
        $chunks = preg_split('@(</?[^>]*>)@i', $text, -1, PREG_SPLIT_DELIM_CAPTURE);
        // Find block level elements, and check the next chunk
        $block_elements = array('<p>', '<ol>', '<ul>', '<li>', '<div>');
        $i = 0;
        $output = '';
        while ($i < count($chunks)) {
            if (in_array($chunks[$i], $block_elements)) {
                $text_chunk = $this->textChuck($chunks, $i+1);
                if ($text_chunk) {
                    $dir = $this->determineDir($chunks[$text_chunk]);
                    $chunks[$i] = '<' . trim($chunks[$i], '<>')  . ' style="direction:' . $dir . '">';
                    $output .= $chunks[$i];
                }
            }
            else {
                $output .= $chunks[$i];
            }
            $i++;
        }

        return new FilterProcessResult($output);
    }

    /**
     * {@inheritdoc}
     */
    public function tips($long = FALSE) {
        return $this->t('Paragraphs directionality will be automatically set to RTL or LTR.');
    }

    /**
     * @param $chunks
     * @param $i
     * @return mixed
     */
    private function textChuck($chunks, $i) {
        // If the current chunk is a valid text, return it, otherwise, keep digging.
        if (preg_match('@^<[^>]*>@u', $chunks[$i]) || empty($chunks[$i])) {
            if (isset($chunks[$i+1])) {
                $next = _pdir_text_chuck($chunks, $i+1);
                return $next;
            }
        }
        else {
            return $i;
        }
    }

    /**
     * @param $chunk
     * @return string
     */
    private function determineDir($chunk) {
        // Check wehther the first alphabethic character is an RTL one (e.g., belongs to an RTL language)
        // The following line means: anything which is NOT a Letter, or digit. Pattern strings are treated as UTF-8.
        // See: http://il2.php.net/manual/en/regexp.reference.php and http://il2.php.net/manual/en/reference.pcre.pattern.modifiers.php for details.
        $neutral_chars = array('@\PL@u', '@\d@u');
        $chunk = preg_replace($neutral_chars, '', $chunk);
        if (preg_match('@(^[\x{0590}-\x{05f4}])@u', $chunk) || // Hebrew
            preg_match('@(^[\x{0600}-\x{06ff}])@u', $chunk) || // Arabic
            preg_match('@(^rlm)@u', $chunk)) { // &rlm; HTML entity (with & and ; filtered out)
            return 'rtl';
        }
        else return 'ltr';
    }
}